package com.blamejared.compat.botania.handlers;

import java.util.Iterator;
import crafttweaker.CraftTweakerAPI;
import java.util.Collection;
import com.blamejared.mtlib.helpers.StackHelper;
import java.util.LinkedList;
import java.util.List;
import java.util.Collections;
import com.blamejared.mtlib.utils.BaseListRemoval;
import com.blamejared.mtlib.helpers.LogHelper;
import vazkii.botania.api.BotaniaAPI;
import com.blamejared.mtlib.utils.BaseListAddition;
import stanhebben.zenscript.annotations.ZenMethod;
import vazkii.botania.api.recipe.RecipeManaInfusion;
import com.blamejared.mtlib.helpers.InputHelper;
import com.blamejared.ModTweaker;
import crafttweaker.api.item.IIngredient;
import crafttweaker.api.item.IItemStack;
import crafttweaker.annotations.ZenRegister;
import crafttweaker.annotations.ModOnly;
import stanhebben.zenscript.annotations.ZenClass;

@ZenClass("mods.botania.ManaInfusion")
@ModOnly("botania")
@ZenRegister
public class ManaInfusion
{
    protected static final String name = "Botania Mana Infusion";
    
    @ZenMethod
    public static void addInfusion(final IItemStack output, final IIngredient input, final int mana) {
        ModTweaker.LATE_ADDITIONS.add(new Add(new RecipeManaInfusion(InputHelper.toStack(output), InputHelper.toObject(input), mana)));
    }
    
    @ZenMethod
    public static void addAlchemy(final IItemStack output, final IIngredient input, final int mana) {
        final RecipeManaInfusion recipe = new RecipeManaInfusion(InputHelper.toStack(output), InputHelper.toObject(input), mana);
        recipe.setCatalyst(RecipeManaInfusion.alchemyState);
        ModTweaker.LATE_ADDITIONS.add(new Add(recipe));
    }
    
    @ZenMethod
    public static void addConjuration(final IItemStack output, final IIngredient input, final int mana) {
        final RecipeManaInfusion recipe = new RecipeManaInfusion(InputHelper.toStack(output), InputHelper.toObject(input), mana);
        recipe.setCatalyst(RecipeManaInfusion.conjurationState);
        ModTweaker.LATE_ADDITIONS.add(new Add(recipe));
    }
    
    @ZenMethod
    public static void removeRecipe(final IIngredient output) {
        ModTweaker.LATE_REMOVALS.add(new Remove(output));
    }
    
    private static class Add extends BaseListAddition<RecipeManaInfusion>
    {
        public Add(final RecipeManaInfusion recipe) {
            super("Botania Mana Infusion", BotaniaAPI.manaInfusionRecipes);
            this.recipes.add(recipe);
        }
        
        public String getRecipeInfo(final RecipeManaInfusion recipe) {
            return LogHelper.getStackDescription((Object)recipe.getOutput());
        }
    }
    
    private static class Remove extends BaseListRemoval<RecipeManaInfusion>
    {
        final IIngredient output;
        
        public Remove(final IIngredient output) {
            super("Botania Mana Infusion", BotaniaAPI.manaInfusionRecipes, (List)Collections.emptyList());
            this.output = output;
        }
        
        public String getRecipeInfo(final RecipeManaInfusion recipe) {
            return LogHelper.getStackDescription((Object)recipe.getOutput());
        }
        
        public void apply() {
            final List<RecipeManaInfusion> recipes = new LinkedList<RecipeManaInfusion>();
            for (final RecipeManaInfusion r : BotaniaAPI.manaInfusionRecipes) {
                if (r.getOutput() != null && StackHelper.matches(this.output, InputHelper.toIItemStack(r.getOutput()))) {
                    recipes.add(r);
                }
            }
            if (!recipes.isEmpty()) {
                this.recipes.addAll(recipes);
                super.apply();
            }
            else {
                LogHelper.logWarning(String.format("No %s Recipe found for %s. Command ignored!", "Botania Mana Infusion", this.output.toString()));
            }
            CraftTweakerAPI.getLogger().logInfo(super.describe());
        }
        
        public String describe() {
            return "Attempting to remove mana infusion recipe for " + this.output.getItems();
        }
    }
}
